#ifndef BOX_H
#define BOX_H

#include <iostream>
#include <map>
#include <vector>

#include "sphere.h"

class Box
{
public:
   void in(std::istream* source);  // read from *source
   void out(std::ostream* target);  // write to *target
   
   long count_overlaps();  // count number of overlaps (if inner shielding is broken, it counts as eight overlaps)
   
   void set_extension_global(double x, double y, double z) {
      this->extension_global[0] = x;
      this->extension_global[1] = y;
      this->extension_global[2] = z;
   }
   void set_halo(double h) { this->halo_size = h; }
   
   void set_boxes_global(const int ibg[3]) {
      for(int d=0; d < 3; d++) this->boxes[d] = ibg[d];
   }
   void set_box_rank(const int ibr[3]) {
      for(int d=0; d < 3; d++) this->boxrank[d] = ibr[d];
   }
   
   int add_component(double sphere_size);  // create component, return cid
   int add_particle(size_t pid, double sphere_size, double q[3]);  // create particle, return cid
   
private:
   double extension_global[3];  // size of the global system volume
   
   double ext_local_min[3];  // lower corner of the local box (core)
   double ext_local_max[3];  // upper corner of the local box (core)
   double halo_size = 0;  // size of the halo
   
   int boxes[3] = {1, 1, 1};
   int boxrank[3] = {0, 0, 0};
   
   std::map<double, int, std::greater<double>> components;  // component IDs (value) ordered by size (key)
   std::vector<std::vector<Sphere>> particles;  // maps each component ID to the spheres associated with that component
   
   size_t N_global = 0;  // total number of particles

   size_t N_local_core = 0;  // local number of particles
   size_t N_local_halo = 0;  // local number of particles
};

std::istream& operator>>(std::istream& is, Box& b);
std::ostream& operator<<(std::ostream& os, Box& b);

#endif
